@@ -9,6 +9,7 @@ from __future__ import unicode_literals
 import copy
 import itertools
 import json
+from logging import getLogger
 import re
 from msrest.serialization import Model
 try:
@@ -20,7 +21,7 @@ from . import errors
 from . import _pool_utils as pool_utils
 from . import models
 
-
+logger = getLogger(__name__)
 try:
     _UNICODE_TYPE = unicode
 except NameError:
@@ -215,7 +216,7 @@ def _add_cmd_prefix(task, os_flavor):
     if os_flavor == pool_utils.PoolOperatingSystemFlavor.WINDOWS:
         # TODO: Do we need windows shell escaping?
         task.command_line = 'cmd /c "{}"'.format(task.command_line) #.replace('\"','\\\\\"')
-    if os_flavor == pool_utils.PoolOperatingSystemFlavor.LINUX:
+    elif os_flavor == pool_utils.PoolOperatingSystemFlavor.LINUX:
         task.command_line = '/bin/bash -c \'set -e; set -o pipefail; {}; wait\''.format(task.command_line)
     else:
         raise ValueError("Unknown pool OS flavor: " + str(os_flavor))
@@ -381,15 +382,20 @@ def _validate_parameter(name, content, value):
             value = _validate_bool(value)
         elif content['type'] == 'string':
             value = _validate_string(value, content)
+        elif content['type'] == 'object':
+            pass
+        else:
+            raise ValueError("The parameter '{}' specifies an unsupported "
+                             "type: {}".format(name, content['type']))
         if value not in content.get('allowedValues', [value]):
             raise ValueError("Allowed values: {}".format(', '.join(content['allowedValues'])))
     except TypeError:
         raise TypeError("The value '{}' of parameter '{}' is not a {}".format(
-            name, value, content['type']))
+            value, name, content['type']))
     except ValueError as value_error:
         raise ValueError(
             "The value '{}' of parameter '{}' does not meet the requirement: {}".format(
-                name, value, str(value_error)))
+                value, name, str(value_error)))
     else:
         return value
 
@@ -409,7 +415,10 @@ def _get_template_params(template, param_values):
                 # ARM: '<PropertyName>' : { 'value' : '<PropertyValue>' }
                 # Dictionary: '<PropertyName>' : <PropertyValue>'
                 value = param_values[param]
-                param_keys[param] = value.get('value') if isinstance(value, dict) else value
+                if isinstance(value, dict) and value.get('value') != None:
+                    param_keys[param] = value.get('value')
+                else:
+                    param_keys[param] = value
             except KeyError:
                 param_keys[param] = values.get('defaultValue')
     except KeyError:
@@ -444,23 +453,31 @@ def _parse_arm_parameter(name, template_obj, parameters):
         # Parse nested object if exists
         if len(name) > param_name_end+1:
             param_def = _parse_nested_object(
-                param_def,
+                param_def.get('defaultValue', param_def),
                 name[param_name_end+1:],
                 template_obj,
                 parameters)
-
     except KeyError:
-        raise ValueError("Template does not define parameter '{}'".format(param_name))
-    user_value = param_def.get('defaultValue')
+        if param_def.get('type') != 'object':
+            raise ValueError("Template does not define parameter '{}'".format(param_name))
+
+    user_value = param_def.get('defaultValue') if isinstance(param_def, dict) else param_def
     if parameters and param_name in parameters:
         # Support both ARM and dictionary syntax
         # ARM: '<PropertyName>' : { 'value' : '<PropertyValue>' }
         # Dictionary: '<PropertyName>' : <PropertyValue>'
-        user_value = parameters[param_name]
         try:
-            user_value = user_value['value']
-        except TypeError:
+            parameters_value = parameters[param_name]
+            if len(name) > param_name_end + 1:
+                parameters_value = _parse_nested_object(
+                    parameters_value,
+                    name[param_name_end + 1:],
+                    template_obj,
+                    parameters)
+            user_value = parameters_value
+        except (TypeError, KeyError):
             pass
+
     if user_value is None:
         raise errors.MissingParameterValue(
             "No value supplied for parameter '{}' and no default value".format(param_name),
@@ -471,17 +488,12 @@ def _parse_arm_parameter(name, template_obj, parameters):
         # additional parameter substitutions
         return _parse_template(json.dumps(user_value), template_obj, parameters)
     try:
-        if param_def['type'] == 'int':
-            return _validate_int(user_value, param_def)
-        if param_def['type'] == 'bool':
-            return _validate_bool(user_value)
-        if param_def['type'] == 'string':
-            return _validate_string(user_value, param_def)
+        if not isinstance(param_def, dict):
+            return param_def
+        return _validate_parameter(param_name, param_def, user_value)
     except TypeError:
         raise TypeError("Value '{}' for parameter '{}' must be a {}.".format(
             user_value, param_name, param_def['type']))
-    else:
-        raise TypeError("Parameter type '{}' not supported.".format(param_def['type']))
 
 
 def _parse_nested_object(obj, references, template_obj, parameters):
@@ -522,6 +534,8 @@ def _parse_nested_object(obj, references, template_obj, parameters):
 
     while obj_refs:
         ref = obj_refs.pop(0)
+        if isinstance(ret_obj, list):
+            ref = int(ref)
         ret_obj = ret_obj[ref]
 
     return ret_obj
@@ -846,7 +860,7 @@ def _transform_repeat_task(task, context, index, transformer):
     try:
         for resource in new_task.resource_files:
             _replacement_transform(transformer, resource, 'file_path', context)
-            _replacement_transform(transformer, resource, 'blob_source', context)
+            _replacement_transform(transformer, resource, 'http_url', context)
             try:
                 for param in ['file_group', 'prefix', 'container_url', 'url']:
                     _replacement_transform(transformer, resource.source, param, context)
@@ -935,8 +949,7 @@ def _expand_task_per_file(factory, fileutils):
     try:
         factory.merge_task.id = 'merge'
         factory.merge_task.depends_on = models.TaskDependencies(
-            task_id_ranges=[
-                models.TaskIdRange(start=0, end=len(task_objs) - 1)])
+            task_id_ranges=[models.TaskIdRange(start=0, end=len(task_objs) - 1)])
         task_objs.append(_transform_merge_task(factory.merge_task))
     except AttributeError:  # No merge task
         pass
@@ -1200,3 +1213,31 @@ def validate_json_object(json_obj, obj):
         else:
             inner_type = getattr(obj, key_attr_map[item.lower()])
             validate_json_object(json_obj[item], inner_type)
+
+
+def convert_blob_source_to_http_url(obj):
+    if isinstance(obj, list):
+        out = []
+        for i in obj:
+            out.append(convert_blob_source_to_http_url(i))
+        return out
+    if isinstance(obj, dict):
+        for key in obj:
+            if key in ['resourceFiles', 'commonResourceFiles']:
+                obj[key] = \
+                    [_convert_blob_source_to_http_url(resource_file) for resource_file in obj[key]]
+            obj[key] = convert_blob_source_to_http_url(obj[key])
+    return obj
+
+
+def _convert_blob_source_to_http_url(resource_file):
+    if 'blobSource' in resource_file:
+        if 'filePath' not in resource_file:
+            raise ValueError('Malformed ResourceFile: \'blobSource\' must '
+                             'also have \'file_path\' attribute')
+        resource_file['httpUrl'] = resource_file.pop('blobSource', None)
+        logger.warning('BlobSource has been updated to HttpUrl to reflect new '
+                       'functionality of accepting any http url instead of just'
+                       ' storage blobs. Please update your templates to'
+                       ' reflect this.')
+    return resource_file
